@@ -1,6 +1,6 @@
 # -*- coding:utf-8 -*-
 
-
+import re
 import os
 import sys
 import shutil
@@ -57,13 +57,13 @@ class BflbMcuTool(object):
         self.eflash_loader_cfg = os.path.join(app_path, chipname, "eflash_loader/eflash_loader_cfg.conf")
         self.eflash_loader_cfg_tmp = os.path.join(app_path, chipname, "eflash_loader/eflash_loader_cfg.ini")
         self.eflash_loader_bin = os.path.join(app_path, chipname, "eflash_loader/eflash_loader_40m.bin")
-        self.img_create_path = os.path.join(app_path, chipname, "img_create2")
+        self.img_create_path = os.path.join(app_path, chipname, "img_create_mcu")
         self.efuse_bh_path = os.path.join(app_path, chipname, "efuse_bootheader")
         self.efuse_bh_default_cfg = os.path.join(app_path, chipname, "efuse_bootheader") + "/efuse_bootheader_cfg.conf"
         self.efuse_bh_default_cfg_dp = os.path.join(app_path, chipname, "efuse_bootheader") + "/efuse_bootheader_cfg_dp.conf"
-        self.img_create_cfg_org = os.path.join(app_path, chipname, "img_create") + "/img_create_cfg.conf"
-        self.img_create_cfg_dp_org = os.path.join(app_path, chipname, "img_create") + "/img_create_cfg_dp.conf"
-        self.img_create_cfg = os.path.join(app_path, chipname, "img_create2") + "/img_create_cfg.ini"
+        self.img_create_cfg_org = os.path.join(app_path, chipname, "img_create_mcu") + "/img_create_cfg.conf"
+        self.img_create_cfg_dp_org = os.path.join(app_path, chipname, "img_create_mcu") + "/img_create_cfg_dp.conf"
+        self.img_create_cfg = os.path.join(app_path, chipname, "img_create_mcu") + "/img_create_cfg.ini"
         if not os.path.exists(self.img_create_path):
             os.makedirs(self.img_create_path)
         if os.path.isfile(self.eflash_loader_cfg_tmp) is False:
@@ -290,19 +290,19 @@ class BflbMcuTool(object):
                 if values["img_type"] == "SingleCPU":
                     bootinfo_file = self.img_create_path + "/bootinfo.bin"
                     img_file = self.img_create_path + "/img.bin"
-                    img_output_file = self.img_create_path + "/wholeimg_flash.bin"
+                    img_output_file = self.img_create_path + "/whole_img.bin"
                 elif values["img_type"] == "BLSP_Boot2":
                     bootinfo_file = self.img_create_path + "/bootinfo_blsp_boot2.bin"
                     img_file = self.img_create_path + "/img_blsp_boot2.bin"
-                    img_output_file = self.img_create_path + "/wholeimg_blsp_boot2_flash.bin"
+                    img_output_file = self.img_create_path + "/whole_img_blsp_boot2.bin"
                 elif values["img_type"] == "CPU0":
                     bootinfo_file = self.img_create_path + "/bootinfo_cpu0.bin"
                     img_file = self.img_create_path + "/img_cpu0.bin"
-                    img_output_file = self.img_create_path + "/wholeimg_cpu0_flash.bin"
+                    img_output_file = self.img_create_path + "/whole_img_cpu0.bin"
                 elif values["img_type"] == "CPU1":
                     bootinfo_file = self.img_create_path + "/bootinfo_cpu1.bin"
                     img_file = self.img_create_path + "/img_cpu1.bin"
-                    img_output_file = self.img_create_path + "/wholeimg_cpu1_flash.bin"
+                    img_output_file = self.img_create_path + "/whole_img_cpu1.bin"
                 if values["img_type"] == "SingleCPU":
                     dummy_data = bytearray(8192)
                 else:
@@ -798,6 +798,18 @@ class BflbMcuTool(object):
                 return error
             else:
                 if values["img_file"] == "":
+                    if values["device_tree"]:
+                        ro_params_d = values["device_tree"]
+                        try:
+                            dts_hex = bl_ro_device_tree.bl_dts2hex(ro_params_d)
+                            dts_bytearray = bflb_utils.hexstr_to_bytearray(dts_hex)
+                        except Exception as e:
+                            dts_bytearray = None
+                        tlv_bin = self.img_create_path + "/tlv.bin"
+                        with open(tlv_bin, "wb") as fp:
+                            fp.write(dts_bytearray)
+                        error = "tvl bin created"
+                        return error
                     if values["dl_chiperase"] == "True":
                         bflb_utils.printf("flash chiperase operation")
                         return True
@@ -814,6 +826,9 @@ class BflbMcuTool(object):
                             dts_bytearray = bflb_utils.hexstr_to_bytearray(dts_hex)
                         except Exception as e:
                             dts_bytearray = None
+                        tlv_bin = self.img_create_path + "/tlv.bin"
+                        with open(tlv_bin, "wb") as fp:
+                            fp.write(dts_bytearray)
                         img_org = values["img_file"]
                         if parse_rfpa(img_org) == b'BLRFPARA' and dts_bytearray:
                             length = len(dts_bytearray)
@@ -1432,7 +1447,7 @@ class BflbMcuTool(object):
                                 options.extend(["--efuse",\
                                 "--createcfg=" + self.img_create_cfg])
                                 self.efuse_load_en = True
-            ret = bflb_img_create.compress_dir(self.chipname, "img_create2", self.efuse_load_en)
+            ret = bflb_img_create.compress_dir(self.chipname, "img_create_mcu", self.efuse_load_en)
             if ret is not True:
                 return bflb_utils.errorcode_msg()
             args = parser_eflash.parse_args(options)
@@ -1556,14 +1571,15 @@ def get_value(args):
     
 
 def run():  
-    WINDOWS = sys.platform.startswith("win")
     port = None
+    ports = []
     for item in get_serial_ports():
-        port = item["port"]
-        if WINDOWS and port.startswith("COM") and len(port) > 4:
-            port = "\\\\.\\%s" % port
-        else:
-            port = port  
+        ports.append(item["port"])
+    if ports:
+        try:
+            port = sorted(ports, key=lambda x: int(re.match('COM(\d+)', x).group(1)))[0]
+        except Exception:
+            port = sorted(ports)[0]
     firmware_default = os.path.join(app_path, "img/project.bin")
     parser = argparse.ArgumentParser(description='bflb mcu tool')
     parser.add_argument('--chipname', required=True, help='chip name')
@@ -1584,7 +1600,7 @@ def run():
     if not args.port:
         bflb_utils.printf("Serial port is not found")
     else:
-        bflb_utils.printf("Serial port is " + port) 
+        bflb_utils.printf("Serial port is " + str(port)) 
     bflb_utils.printf("Baudrate is " + str(args.baudrate)) 
     bflb_utils.printf("Firmware is " + args.firmware)
     config = get_value(args)      
@@ -1592,7 +1608,12 @@ def run():
     bflb_utils.printf("==================================================")
     try:
         obj_mcu.create_img(args.chipname, chip_dict[args.chipname], config)
-        if not args.build:
+        if args.build:
+            obj_mcu.bind_img(config)
+            f_org = os.path.join(app_path, args.chipname, "img_create_mcu", "whole_img.bin")
+            f = "firmware.bin"
+            shutil.copy(f_org, f)
+        else:
             obj_mcu.program_img_thread(config)
     except Exception as e:
         error = str(e)
